#!/usr/bin/perl -w

# rec.pl
# michael dungan - mpd@rochester.rr.com
#
# some recipe thing i made because all the
# existing ones suck ass and require
# things like mysql for some reason.
# do you really think the average user of
# software like this has mysql installed?
#
use strict;
use Tk;
use Tk::Dialog;
use Tk::FileDialog;
use File::Basename;
use Getopt::Std;
use Recipe 0.5;

#----------
# globals
#----------
my $VER = 1.0;
my $RCSID = '$Id: rec.pl,v 1.18 2002/04/04 05:06:10 vega Exp $';

#----------
# main window - just easier to leave it global
#----------
my $main;			# main window
my $menubar;		# upper menubar
my $filebutton;		# "File" in menubar
my $filemenu;		# $filemenus menu
my $recipebutton;	# "Recipe" in menubar
my $recipemenu;		# $recipes menu
my $helpbutton;		# "Help" in menubar
my $helpmenu;		# $helpmenus menu
my $recipe_lb;		# listbox for recipes
my $buttonbar;		# frame for buttonbox below listbox
my $new_button;		# "New" button in buttonbar
my $open_button;	# "Open" button in buttonbar
my $edit_button;	# "Edit" button in buttonbar
my $save_button;	# "Save" button in buttonbar
my $view_button;	# "View" button in buttonbar
my $delete_button;	# "Delete" button in buttonbar
my $status;			# Status bar at bottom

#----------
# misc - these should eventually be put into a cookbook object
#----------
my %cookbook;					# current cookbook in memory.
my $cb_path = './';				# sane default path to cookbook file
my $cb_file = 'cookbook.rec';	# default name
my $cb = $cb_path.$cb_file;		# full pathname of current cookbook
my $need_to_save;				# check on exit - do we need to save cookbook?

my %arguments;					# hash for getopts storage

&create_mainwindow();

getopts('hf:', \%arguments);

$arguments{h} && &usage();

# were we passed a file on the command line?
# this will override the REC_BOOKBOOK env var
if($arguments{f}) {
	my $new_cb = $arguments{f};
	if( -r $new_cb ) {
		$cb = $new_cb;
		$cb_path = dirname($cb);
		$cb_file = basename($cb);
		&open_cookbook;
	}
}
# check environment variable for default cookbook
elsif($ENV{REC_COOKBOOK}) {
	my $new_cb = $ENV{REC_COOKBOOK};
	if( -r $new_cb ) {
		$cb = $new_cb;
		$cb_path = dirname($cb);
		$cb_file = basename($cb);
		&open_cookbook;
	}
}

#----------
# begin MainLoop
#----------
MainLoop;

# end "main"

sub usage {
	print STDERR "Usage: rec.pl [-h] [-f FILENAME]\n\n".
	             "\t-h - Print this message and exit\n".
	             "\t-f FILENAME - Cookbook to open at start. ".
	             "Overrides the REC_COOKBOOK\n\t              ".
	             "environment variable.\n".
	             "\n\n";
	exit;
}

# button subroutines
sub exit_choice {

	if($need_to_save) {
		my $dialog_delete = $main->Dialog(
			-title		=> 'Save cookbook?',
			-text		=> 'Do you wish to save your cookbook?',
			-default_button => 'Cancel',
			-buttons	=> ['Yes', 'No', 'Cancel']);
	
		my $yn = $dialog_delete->Show();
		if($yn eq 'Cancel') {
			$status->configure(-text=>'Quit canceled.');
			return;
		}
		&save_choice if $yn eq 'Yes';
	}

    exit;
}

sub new_choice {
    $status->configure(-text=>'New file.');

	&create_newrecipewindow();
}

sub open_choice {
	$status->configure(-text=>'Open cookbook.');
	my $open_dialog = $main->FileDialog(-Title=>'Open Cookbook',
	                                    -Create=>0,
	                                    -FPat=>'*.rec',
	                                    -Path=>$cb_path,
	                                    -File=>$cb_file,
	                                    -OKButtonLabel=>'Open',
	                                    -ShowAll=>'YES');

	$cb = $open_dialog->Show();
	$cb_path = dirname($cb) if $cb;
	$cb_file = basename($cb) if $cb;

	# check if 'cancel' pressed
	unless(defined($cb)) {
		$status->configure(-text=>'Open canceled.');
		return;
	}

	&open_cookbook;
}

#----------
# open_cookbook
# call this once we know the name of the book we want to open
# args: none
#----------
sub open_cookbook {
	# read in cookbook.
	my $tmp_title = undef;
	my $tmp_serves = undef;
	my @tmp_ings;
	my @tmp_dirs;

	my $cmd;			# the first part of the line.
	my $line;			# the rest of the line

	my $newrec;			# the recipe we've just read in

	return unless(open(CB, $cb));

	#get rid of what's there now.
	&delete_all();

	while(<CB>) {

		next if /^\s+$/;

		chomp;

		($cmd, $line) = split /:\t/;

		# are we at the end of this recipe?
		if( $cmd =~ /\w*---END\w*/ ) {

			# need a real title to hash correctly.
			# don't destroy window, though, in case user can repent.
			if(!defined($tmp_title) || $tmp_title =~ /^\s*$/) {
				&error_dialog('No title defined. Recipe not added.');
				undef $tmp_title;
				undef $tmp_serves;
				undef @tmp_ings;
				undef @tmp_dirs;
				next;
			}

			# create and save new recipe
			$newrec = new Recipe();
			$newrec->title($tmp_title);
			$newrec->serves($tmp_serves);
			$newrec->ingredients(@tmp_ings);
			$newrec->directions(@tmp_dirs);
			$cookbook{$tmp_title} = $newrec;

			# blanking these are necessary
			undef $tmp_title;
			undef $tmp_serves;
			undef @tmp_ings;
			undef @tmp_dirs;
			next;
		}

		# strip any leading or trailing whitespace from $line
		for($line) {
			s/^\s+//;
			s/\s+$//;
		}

		if( $cmd =~ /\w*title\w*/ ) {
			&error_dialog('second title found: $tmp_title')
				if defined($tmp_title);
			$tmp_title = $line;
		}
		elsif( $cmd =~ /\w*serves\w*/ ) {
			&error_dialog('second serves found: $tmp_serves')
				if defined($tmp_serves);
			$tmp_serves = $line;
		}
		elsif( $cmd =~ /\w*ing\w*/ ) {
			push(@tmp_ings, $line);
		}
		elsif( $cmd =~ /\w*step\w*/ ) {
			push(@tmp_dirs, $line);
		}
	}
	close CB or &error_dialog("Can't close $cb: $!");

    $status->configure(-text=>"$cb_file opened.");
	$main->configure(-title=>"rec $VER - $cb_file");

	$need_to_save = 0;

	&refresh_main();
}

sub save_choice {
    $status->configure(-text=>'Saving...');
	my $recipe;

	my $save_dialog = $main->FileDialog(-Title=>'Save Cookbook',
	                                    -FPat=>'*.rec',
	                                    -Path=>$cb_path,
	                                    -File=>$cb_file,
	                                    -OKButtonLabel=>'Save',
	                                    -ShowAll=>'YES');

	$cb = $save_dialog->Show();
	$cb_path = dirname($cb) if $cb;
	$cb_file = basename($cb) if $cb;

	unless(open(OUTFILE, ">$cb")) {
		&error_dialog("Can't open $cb: $!\n");
		$status->configure(-text=>"Can't open $cb: $!");
		return;
	}

	foreach (sort keys %cookbook) {
		$recipe = $cookbook{$_};
		print OUTFILE "title:\t".$recipe->title."\n";
		print OUTFILE "serves:\t".$recipe->serves."\n";
		foreach my $ing ($recipe->ingredients) {
			print OUTFILE "ing:\t$ing\n";
		}
		foreach my $dir ($recipe->directions) {
			print OUTFILE "step:\t$dir\n";
		}
		print OUTFILE "---END\n";
	}
	close OUTFILE or &error_dialog("Can't close output file: $!");
	$status->configure(-text=>"Cookbook saved to $cb.");
	$need_to_save = 0;
}

sub about_choice {   
    $status->configure(-text=>"About rec $VER");
	my $msg = "rec version $VER\n\ncopyright 2002 - mpd\n".
	          'mpd@rochester.rr.com';

	my $dialog_about = $main->messageBox(
		-title		=> "rec - $VER: About",
		-message	=> "$msg",
		-type=>'OK',
		-default=>'OK');
}

sub view_choice {   
    $status->configure(-text=>'View selected recipe(s).');
	my @selected = $recipe_lb->curselection;
	return if @selected <= 0;
	foreach(@selected) {
		&view_recipe($recipe_lb->get($_));
	}
}

sub edit_choice {   
    $status->configure(-text=>'Edit selected recipe(s).');
	my @selected = $recipe_lb->curselection;
	return if @selected <= 0;
	foreach(@selected) {
		&edit_recipe($recipe_lb->get($_));
	}
}

sub delete_choice {   
	$status->configure(-text=>'Delete selected recipe(s).');
	my @selected = $recipe_lb->curselection;
	return if @selected <= 0;

	my $r = 'recipes';
	$r = 'recipe' if @selected == 1;

	my $dialog_delete = $main->Dialog(
		-title		=> 'Delete',
		-text		=> "Delete selected $r?",
		-default_button => 'Cancel',
		-buttons	=> ['Ok', 'Cancel']);

	my $yn = $dialog_delete->Show();

	if($yn eq 'Cancel') {
		$status->configure(-text=>'Delete canceled.');
		return;
	}

	my $title;

	# must be reverse, as deleting from the listbox updates
	# the indices of the other elements immediately, so we
	# need to remove from the bottom->top.
	foreach(@selected) {
		$title = $recipe_lb->get($_);
		next unless $title;
		delete $cookbook{$title};
	}
	# for some reason, this must be done separately.
	# if you put this in foreach above, you WILL get
	# bugs.
	foreach(reverse sort @selected) {
		$recipe_lb->delete($_);
	}

	$r = 'Recipes';
	$r = 'Recipe' if @selected == 1;
	$status->configure(-text=>"$r deleted.");

	&refresh_main;

	$need_to_save = 1;
}

#----------
# view_recipe
# arguments: 0: $title - string representing the title of the recipe to view.
#----------
sub view_recipe
{
	my $title = $_[0];
	my $rec = $cookbook{$title};

	# recipe window
	my $view_recipe_top = $main->Toplevel;
	$view_recipe_top->title($title);

	# title, serves frame
	my $view_title_frame = $view_recipe_top->Frame(
	                                              -relief=>'raised',
	                                              -borderwidth=>2);

	my $view_title_label = $view_title_frame->Label(
	                          -text=>'Title: ');
	my $view_title_entry = $view_title_frame->Entry(
	                          -width=>40,
	                          -background=>'white');
	$view_title_entry->insert(0, $title);
	$view_title_entry->configure(-state=>'disabled');
	
	my $view_serves_label = $view_title_frame->Label(
	                          -text=>'Serves: ');
	my $view_serves_entry = $view_title_frame->Entry(
	                          -width=>4,
	                          -background=>'white');
	# need to insert text before entry is disabled. bummer.
	$view_serves_entry->insert(0, $rec->serves);
	$view_serves_entry->configure(-state=>'disabled');
	
	$view_title_label->pack(-side=>'left');
	$view_title_entry->pack(-side=>'left');

	$view_serves_entry->pack(-side=>'right');
	$view_serves_label->pack(-side=>'right');

	$view_title_frame->pack(-side=>'top', -fill=>'x');

	# listbox frame
	my $view_recipe_frame = $view_recipe_top->Frame(
	                              -relief=>'raised',
	                              -borderwidth=>2);
	
	# Ingredients Textbox
	my $view_ing_tb = $view_recipe_frame->Scrolled('Text',
                                  -scrollbars=>'oe',
                                  -height=>20,
	                              -width=>40,
	                              -wrap=>'word',
	                              -background=>'white'
	                             );

	#insert all the ingredients
	my @ings = $rec->ingredients;
	my $ingstr = join("\n\n", @ings);
	$view_ing_tb->insert('end', $ingstr);
	$view_ing_tb->configure(-state=>'disabled');
	
	# Directions Listbox
	my $view_dir_tb = $view_recipe_frame->Scrolled('Text',
                                  -scrollbars=>'oe',
	                              -height=>20,
	                              -width=>40,
	                              -wrap=>'word',
	                              -background=>'white'
	                             );
	#insert directions
	my @dirs = $rec->directions;
	my $dirstr = join("\n\n", @dirs);
	$view_dir_tb->insert('end', $dirstr);
	$view_dir_tb->configure(-state=>'disabled');

	# pack to the left
	$view_ing_tb->pack(-side=>'left', -expand=>1, -fill=>'both');
	$view_dir_tb->pack(-side=>'left', -expand=>1, -fill=>'both');

	$view_recipe_frame->pack(-side=>'top', -expand=>1, -fill=>'both');

	# Dismiss button frame, w/ button
	my $view_recipe_buttonbar = $view_recipe_top->Frame(-relief=>'raised',
	                                                  -borderwidth=>2);

	my $view_recipe_dismiss_button = $view_recipe_buttonbar->Button(
	                                   text=>'Dismiss',
	                                   command=>sub {
	                                     $view_recipe_top->destroy
	                                   });

#	my $view_recipe_print_button = $view_recipe_buttonbar->Button(
#	                                   text=>'Print',
#	                                   command=>\&view_rec_print
#	                                   );
#	$view_recipe_print_button->pack(-side=>'right');

	$view_recipe_dismiss_button->pack(-side=>'right');

	$view_recipe_buttonbar->pack(-side=>'top',
	                            -fill=>'x');

} # end view_recipe

#----------
# create_mainwindow
#----------
sub create_mainwindow
{
	# main window
	$main = MainWindow->new(-width=>587, -height=>449);
	$main->title("rec - $VER");
	$main->minsize(qw(587 449));
	
	# menubar
	$menubar = $main->Frame(-relief=>'raised',
	    -borderwidth=>2);
	
	# File menu.
	$filebutton = $menubar->Menubutton(-text=>'File',
	    -underline => 0);
	
	# Menus are children of Menubuttons.
	$filemenu = $filebutton->Menu();
	
	# Associate Menubutton with Menu.
	$filebutton->configure(-menu=>$filemenu);
	
	# Create menu choices.
	$filemenu->command(-command => \&open_choice,
	    -label => 'Open',
	    -underline => 0);
	
	$filemenu->command(-command => \&save_choice,
	    -label => 'Save',
	    -underline => 0);
	
	$filemenu->separator;
	
	$filemenu->command(-label => 'Quit',
	    -command => \&exit_choice,
	    -underline => 0); 


	# Recipe menu.
	$recipebutton = $menubar->Menubutton(-text=>'Recipe',
	    -underline => 0);
	
	$recipemenu = $recipebutton->Menu();

	$recipebutton->configure(-menu=>$recipemenu);

	$recipemenu->command(-command => \&new_choice,
	    -label => 'New',
	    -underline => 0);

	$recipemenu->command(-command => \&edit_choice,
	    -label => 'Edit',
	    -underline => 0);
	
	$recipemenu->command(-command => \&delete_choice,
	    -label => 'Delete',
	    -underline => 0);

	$recipemenu->separator;

	$recipemenu->command(-command => \&view_choice,
	    -label => 'View',
	    -underline => 0);
	
	# Help menu.
	$helpbutton = $menubar->Menubutton(-text=>'Help',
	    -underline => 0);

	$helpmenu = $helpbutton->Menu();

	$helpmenu->command(-command => \&about_choice,
	    -label => 'About',
	    -underline => 0);

	$helpbutton->configure(-menu=>$helpmenu);



	# pack to the left
	$filebutton->pack(-side=>'left');
	$recipebutton->pack(-side=>'left');

	# except help.
	$helpbutton->pack(-side=>'right');

	$menubar->pack(-side=>'top', -fill=>'x');


	# listbox
	$recipe_lb = $main->Scrolled('Listbox',
                                 -scrollbars=>'osoe',
                                 -height => 20,
	                             -width => 80,
	                             -selectmode => 'extended'
	                            );

	# bind for double clicking
	$recipe_lb->bind('<Double-ButtonPress-1>' => \&view_choice);

   
	# Set to expand, with padding.
	$recipe_lb->pack(-side=>"top", -expand=>1, -fill=>'both');
 

	# button bar
	$buttonbar = $main->Frame(-borderwidth=>2);
	
	$new_button = $buttonbar->Button(text    => 'New',
	                       command => \&new_choice);
	
	$edit_button = $buttonbar->Button(text    => 'Edit',
	                       command => \&edit_choice);
	
	$open_button = $buttonbar->Button(text    => 'Open',
	                       command => \&open_choice);
	
	$save_button = $buttonbar->Button(text    => 'Save',
	                       command => \&save_choice);
	
	$view_button = $buttonbar->Button(text    => 'View',
	                       command => \&view_choice);
	
	$delete_button = $buttonbar->Button(text    => 'Delete',
	                       command => \&delete_choice);
	
	$delete_button->pack(-side=>'right');
	$view_button->pack(-side=>'right');
	$save_button->pack(-side=>'right');
	$open_button->pack(-side=>'right');
	$edit_button->pack(-side=>'right');
	$new_button->pack(-side=>'right');

	$buttonbar->pack(-fill=>'x');

	# status area
	$status = $main->Label(-text=>"rec - $VER",
	    -relief=>'sunken',
	    -borderwidth=>2,
	    -anchor=>"w");   

	$status->pack(-side=>'top', -fill=>'x');

	# key bindings
	$main->bind('<Control-n>' => \&new_choice);
	$main->bind('<Control-e>' => \&edit_choice);
	$main->bind('<Control-o>' => \&open_choice);
	$main->bind('<Control-s>' => \&save_choice);
	$main->bind('<Control-v>' => \&view_choice);
	$main->bind('<Control-d>' => \&delete_choice);
	$main->bind('<Delete>'    => \&delete_choice);
	$main->bind('<Control-q>' => \&exit_choice);
	
}

#----------
# create_newrecipewindow
#----------
sub create_newrecipewindow
{
	$main->Busy(-recurse=>1);

	# recipe window
	my $new_recipe_top = $main->Toplevel(-width=>587, -height=>429);
	$new_recipe_top->title("rec - New Recipe");
	$new_recipe_top->minsize(qw(587 429));

	# title, serves frame
	my $new_title_frame = $new_recipe_top->Frame(
	                                              -relief=>'raised',
	                                              -borderwidth=>2);
	my $new_title_label = $new_title_frame->Label(
	                          -text=>'Title: ');
	my $new_title_entry = $new_title_frame->Entry(
	                          -width=>40,
	                          -background=>'white');
	
	$new_recipe_top->Advertise('title_entry'=>$new_title_entry);

	my $new_serves_label = $new_title_frame->Label(
	                          -text=>'Serves: ');
	my $new_serves_entry = $new_title_frame->Entry(
	                          -width=>4,
	                          -background=>'white');

	$new_recipe_top->Advertise('serves_entry'=>$new_serves_entry);
	
	$new_title_label->pack(-side=>'left');
	$new_title_entry->pack(-side=>'left');

	$new_serves_entry->pack(-side=>'right');
	$new_serves_label->pack(-side=>'right');

	$new_title_frame->pack(-side=>'top', -fill=>'x');

	# listbox frame
	my $new_recipe_frame = $new_recipe_top->Frame(
	                                              -relief=>'raised',
	                                              -borderwidth=>2);
	
	# Ingredients Textbox
	my $new_ing_tb = $new_recipe_frame->Scrolled('Text',
                                  -scrollbars=>'oe',
                                  -height=>20,
	                              -width=>40,
	                              -wrap=>'word',
	                              -background=>'white'
	                             );
	
	$new_recipe_top->Advertise('ing_box'=>$new_ing_tb);

	# Directions Listbox
	my $new_dir_tb = $new_recipe_frame->Scrolled('Text',
                                  -scrollbars=>'oe',
	                              -height=>20,
	                              -width=>40,
	                              -wrap=>'word',
	                              -background=>'white'
	                             );

	$new_recipe_top->Advertise('dir_box'=>$new_dir_tb);

	# pack to the left
	$new_ing_tb->pack(-side=>'left', -expand=>1, -fill=>'both');
	$new_dir_tb->pack(-side=>'left', -expand=>1, -fill=>'both');

	$new_recipe_frame->pack(-side=>'top', -expand=>1, -fill=>'both');

	# OK button frame, w/ button
	my $new_recipe_buttonbar = $new_recipe_top->Frame(-relief=>'raised',
	                                                  -borderwidth=>2);

	my $new_recipe_ok_button = $new_recipe_buttonbar->Button(text=>'Ok',
	                               command=>[\&new_recipe_ok,
	                                         $new_recipe_top]);

	my $new_recipe_cancel_button = $new_recipe_buttonbar->
	                               Button(text=>'Cancel',
	                               command=>sub {
	                               $status->configure(text=>'Canceled');
	                               $new_recipe_top->destroy();
	                               $main->Unbusy(); });


	$new_recipe_cancel_button->pack(-side=>'right');
	$new_recipe_ok_button->pack(-side=>'right');

	$new_recipe_buttonbar->pack(-side=>'top',
	                            -fill=>'x');
}

#----------
# new_recipe_ok
# what happens when the "ok" button is clicked when created a new recipe.
# arguments: 0: new_recipe_top - the new recipe toplevel - need to destroy.
#----------
sub new_recipe_ok {
	my $new_recipe_top = $_[0];
	my $title = $new_recipe_top->Subwidget('title_entry')->get();

	# we need a real title to hash the recipe. therefore, it's
	# an error if there isn't one present.
	if( $title =~ /^\s*$/ ) {
		$status->configure(-text=>'No title entered. Recipe not saved.');
		return;
	}

	my $recipe = new Recipe();
	$recipe->title($title);

	$recipe->serves($new_recipe_top->Subwidget('serves_entry')->get());

	my $line;
	my @ings;
	my $ing_box = $new_recipe_top->Subwidget('ing_box');
	$line = $ing_box->get('1.0', 'end');
	@ings = split(/\n/, $line);
	$recipe->ingredients(@ings);

	my @dirs;
	my $dir_box = $new_recipe_top->Subwidget('dir_box');
	$line = $dir_box->get('1.0', 'end');
	@dirs = split(/\n/, $line);
	$recipe->directions(@dirs);

	$cookbook{$title} = $recipe;
	$need_to_save = 1;
	&refresh_main;
	$status->configure(text=>"\"$title\" added.");
	$new_recipe_top->destroy();
	$main->Unbusy();
}

sub delete_all {
	undef %cookbook;
	$recipe_lb->delete(0,'end');
}

sub refresh_main {
	$recipe_lb->delete(0,'end');
	foreach(sort keys %cookbook) {
		$recipe_lb->insert('end', $_);
	}
}

sub error_dialog
{
	my $msg = shift;
	return unless $msg;

	my $dialog_error = $main->messageBox(
		-title		=> 'Error',
		-message	=> "$msg",
		-type=>'OK',
		-default=>'OK');
}

sub edit_recipe
{
	my $rec_title = $_[0];
	my $rec = $cookbook{$rec_title};

	# recipe window
	my $edit_recipe_top = $main->Toplevel;
	$edit_recipe_top->title($rec_title);

	# title, serves frame
	my $edit_title_frame = $edit_recipe_top->Frame(
	                                              -relief=>'raised',
	                                              -borderwidth=>2);

	my $edit_title_label = $edit_title_frame->Label(
	                          -text=>'Title: ');
	my $edit_title_entry = $edit_title_frame->Entry(
	                          -width=>40,
	                          -background=>'white');
	$edit_recipe_top->Advertise('title_entry'=>$edit_title_entry);
	$edit_title_entry->insert(0, $rec_title);
	
	my $edit_serves_label = $edit_title_frame->Label(
	                          -text=>'Serves: ');
	my $edit_serves_entry = $edit_title_frame->Entry(
	                          -width=>4,
	                          -background=>'white');

	$edit_recipe_top->Advertise('serves_entry'=>$edit_serves_entry);

	$edit_serves_entry->insert(0, $rec->serves);
	
	$edit_title_label->pack(-side=>'left');
	$edit_title_entry->pack(-side=>'left');

	$edit_serves_entry->pack(-side=>'right');
	$edit_serves_label->pack(-side=>'right');

	$edit_title_frame->pack(-side=>'top', -fill=>'x');

	# listbox frame
	my $edit_recipe_frame = $edit_recipe_top->Frame(
	                              -relief=>'raised',
	                              -borderwidth=>2);
	
	# Ingredients Textbox
	my $edit_ing_tb = $edit_recipe_frame->Scrolled('Text',
                                  -scrollbars=>'oe',
                                  -height=>20,
	                              -width=>40,
	                              -wrap=>'word',
	                              -background=>'white'
	                             );
	$edit_recipe_top->Advertise('ing_box'=>$edit_ing_tb);

	#insert all the ingredients
	my @ings = $rec->ingredients;
	my $ingstr = join("\n", @ings);
	$edit_ing_tb->insert('end', $ingstr);
	
	# Directions Listbox
	my $edit_dir_tb = $edit_recipe_frame->Scrolled('Text',
                                  -scrollbars=>'oe',
	                              -height=>20,
	                              -width=>40,
	                              -wrap=>'word',
	                              -background=>'white'
	                             );
	$edit_recipe_top->Advertise('dir_box'=>$edit_dir_tb);

	#insert directions
	my @dirs = $rec->directions;
	my $dirstr = join("\n", @dirs);
	$edit_dir_tb->insert('end', $dirstr);

	# pack to the left
	$edit_ing_tb->pack(-side=>'left', -expand=>1, -fill=>'both');
	$edit_dir_tb->pack(-side=>'left', -expand=>1, -fill=>'both');

	$edit_recipe_frame->pack(-side=>'top', -expand=>1, -fill=>'both');

	# Dismiss button frame, w/ button
	my $edit_recipe_buttonbar = $edit_recipe_top->Frame(-relief=>'raised',
	                                                  -borderwidth=>2);

	my $edit_recipe_cancel_button = $edit_recipe_buttonbar->Button(
	                                   -text=>'Cancel',
	                                   -command=>sub {
	                                     $status->configure(-text=>
	                                       "Edit \"$rec_title\" canceled."),
	                                     $edit_recipe_top->destroy
	                                   });

	my $edit_recipe_save_button = $edit_recipe_buttonbar->Button(
	                                   -text=>'Save',
	                                   -command=>[\&edit_rec_save,
	                                              $rec_title,
	                                              $edit_recipe_top]
	                                   );

	$edit_recipe_cancel_button->pack(-side=>'right');
	$edit_recipe_save_button->pack(-side=>'right');

	$edit_recipe_buttonbar->pack(-side=>'top',
	                             -fill=>'x');

} # end edit_recipe

#----------
# edit_rec_save - save a freshly edited recipe to the cookbook
# arguments: 0 - the old title of the recipe, to delete from cookbook
#            1 - reference to the toplevel widget, as it needs to be destroyed
#----------
sub edit_rec_save
{
	my $old_title = $_[0];
	my $recipe = $cookbook{$old_title};

	my $edit_recipe_top = $_[1];

	my $title = $edit_recipe_top->Subwidget('title_entry')->get();

	# we need a real title to hash the recipe. therefore, it's
	# an error if there isn't one present.
	if( $title =~ /^\s*$/ ) {
		$status->configure(-text=>'No title entered. Recipe not saved.');
		return;
	}

	delete $cookbook{$old_title};

	$recipe->title($title);

	$recipe->serves($edit_recipe_top->Subwidget('serves_entry')->get());

	my $line;
	my @ings;
	my $ing_box = $edit_recipe_top->Subwidget('ing_box');
	$line = $ing_box->get('1.0', 'end');
	@ings = split(/\n/, $line);
	$recipe->ingredients(@ings);

	my @dirs;
	my $dir_box = $edit_recipe_top->Subwidget('dir_box');
	$line = $dir_box->get('1.0', 'end');
	@dirs = split(/\n/, $line);
	$recipe->directions(@dirs);

	$cookbook{$title} = $recipe;
	$need_to_save = 1;
	&refresh_main;
	$status->configure(text=>"$title saved.");
	$edit_recipe_top->destroy();
}
